<?php
// +----------------------------------------------------------------------
// | 监听外部调用请求
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

namespace app\socket\controller\event;
use think\facade\Config;

class onRequest {
    /**
	 * WS的启动实例
	*/
    public $_ws;
    
    // 构造函数
    public function __construct($ws) {
        $this->_ws = $ws;
    }
    
    /**
     * 统一入口
     * @todo 无
     * @author 小黄牛
     * @version v1.0.11 + 2020.04.27
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    public function run($request, $response) {
        // 跨域配置设置
        if ($this->_ws->config['websocket_api_origin']) $response->header('Access-Control-Allow-Origin', $this->_ws->config['websocket_api_origin']); 
        if ($this->_ws->config['websocket_api_type']) $response->header('Content-Type', $this->_ws->config['websocket_api_type']); 
        if ($this->_ws->config['websocket_api_methods']) $response->header('Access-Control-Allow-Methods', $this->_ws->config['websocket_api_methods']); 
        if ($this->_ws->config['websocket_api_credentials']) $response->header('Access-Control-Allow-Credentials', $this->_ws->config['websocket_api_credentials']); 
        if ($this->_ws->config['websocket_api_headers']) $response->header('Access-Control-Allow-Headers', $this->_ws->config['websocket_api_headers']); 

        $res = $this->auth($request, $response);
        if ($res === true) {
            # 获取路由
            $server = $request->server;
            $path_info = '';
            if (!empty($server['path_info'])) {
                $path_info = $server['path_info'];
            } else if (!empty($server['request_uri'])) {
                $path_info = $server['request_uri'];
            }
            # 路由解析错误
            if (!$path_info) {
                $this->endRequest($request, $response, '503', '路由获取失败，请检查环境配置！');
            } else {
                # 查询接口是否存在
                $class = '\app\socket\controller\api'.str_replace('/','\\', rtrim($path_info, '/'));
                if (!class_exists($class)) {
                    $this->endRequest($request, $response, '404', '该接口不存在！');
                } else {
                    # 把本身存在类里
                    $response->class = $this;
                    # 指向该接口
                    $obj = new $class();
                    $obj->run($this->_ws, $request, $response);
                    # 释放内存
                    unset($obj);
                }
            }
        }
    }
    
    /**
     * 鉴权
     * @todo 无
     * @author 小黄牛
     * @version v1.0.9 + 2020.04.16
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    public function auth($request, $response) {
        $header = $request->header;
        $config = Config::pull('laf');
       
        if (!isset($header['api_key'])) {
            $this->endRequest($request, $response, '503', '请携带密钥！');
            return false;
        }

        if ($header['api_key'] != $config['websocket_api_key']) {
            $this->endRequest($request, $response, '503', '密钥错误！');
            return false;
        }

        return true;
    }

    /**
     * 获取请求参数
     * @todo 无
     * @author 小黄牛
     * @version v1.0.9 + 2020.04.16
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    public function param($request) {
        if (!empty($request->post)) return $request->post;
        if (!empty($request->get)) return $request->get;
        return false;
    }

    /**
     * 回调输出接口
     * @todo 无
     * @author 小黄牛
     * @version v1.0.9 + 2020.04.16
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    public function endRequest($request, $response, $code, $msg, $data=[]) {
        $json = [
			'code' => "$code",
            'msg'  => "$msg",
            'data' => $data
		];
		# 输出响应
		$return = json_encode($json, JSON_UNESCAPED_UNICODE);
		# 需要end事件，否注会报500错误，并无结果返回
		# 不知道为啥，CLI模式下这个事件一次请求会有2次监听，但发现最后一次其中的server->request_uri会有个/favicon.ico参数
		# 所以凭借这个参数，我们可以做判断，放弃掉第一次监听返回
		# 还有，如果我们直接在onRequest中过滤掉第一次监听，那第二次监听就不会执行，也会报500错误
		# 所以我们只能在返回的时候做下手脚
		//if($request->server['request_uri'] == '/favicon.ico') {
			$response->end($return);
		//}
		# 而且我发现经过这样处理，onRequest事件那边也只会有一次请求了，特别奇怪。
		# 而且这样返回之后，浏览器直接请求还是报500错误。
		# 熟悉Swoole的朋友可以在下方留言，指教下我的疑惑。
    }
}
