<?php
// +----------------------------------------------------------------------
// | 分表存储引擎
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------
namespace org;

class SubTable
{
    // 缓存表名称地址
    protected static $table_path = '';
    // 缓存表名称列表
    protected static $table_list = [];

    // 表前缀
    private static $prefix;
    // 表中缀：TP的分表链接只能是下划线中缀
    private static $in_prefix;
    // 表名称
    private static $table   = '';
    // 单例实例
    private static $instance;
    // Db实例
    private static $Db;
    
	// 私有化构造函数，可以保证在类外该类不能被实例化
    private function __construct(){}
        
    // 禁止克隆单例类，防止二次new
    private function __clone()
    { 
        die('此对象不允许被克隆');
    }
    
    /**
     * 用于初始化单例类
     * @todo 无
     * @author 小黄牛
     * @version v1.0.1 + 2019.05.23
     * @deprecated 暂不弃用
     * @global 无
     * @param string $table
     * @return void
    */
    public static function init($table, $Db=null, $prefix='tp_', $in_prefix='_')
    {
        self::$table = $table;
        self::$Db = $Db;
        self::$prefix = $prefix;
        self::$in_prefix = $in_prefix;
        if (!self::$instance instanceof self) {
            // 生成缓存地址
            self::$table_path = ROOT_PATH . DIRECTORY_SEPARATOR .'public'. DIRECTORY_SEPARATOR .'data'. DIRECTORY_SEPARATOR .'table_cache' . DIRECTORY_SEPARATOR . $table . '_cache.php';
            // 获得单例
            self::$instance = new self();
            // 返回单例
            return self::$instance;
        }

        return self::$instance;
    }
    
    /**
     * 简单hash算法【改求余数】，生成表名
     * @todo 无
     * @author 小黄牛
     * @version v1.0.1 + 2020.03.09
     * @deprecated 暂不弃用
     * @global 无
     * @param string $channel 查询关键词条件ID，一定要是唯一的值
     * @return string
    */
    public function hash($channel)
    {
        // 求余 100
        $array = explode('.', $channel);
        $hash = $array[0] % 100;
        /*
        $str = crc32($channel);
        if ($str < 0) {
            $hash = "0".substr(abs($str), 0, 1);
        } else {
            $hash = substr($str, 0, 2);
        }
        */

        $table = self::$table . self::$in_prefix . $hash;
        
        return $this->create_table($table);
    }

    /**
     * 对所有表，包括主表进行字段(索引)新增、修改，注意：表名称也使用{table}代替
     * @todo 无
     * @author 小黄牛
     * @version v1.0.1 + 2019.05.20
     * @deprecated 暂不弃用
     * @global 无
     * @param string $sql
     * @return bool
    */
    public function update($sql) {
        // 获取缓存列表
        $this->get_table_list();
        $list = self::$table_list;
        // 也加入主表一起更新
        $list[] = self::$table;
        // 强制删除末尾;号，防止SQL不规范
        $sql = rtrim($sql, ';');

        $sql_list = '';
        // 循环创建SQL
        foreach ($list as $v) {
            $sql_list .= str_replace('{table}', self::$prefix.$v, $sql).';';
        }

        // 批量执行SQL
        $res = $this->exec($sql_list);
        if ($res !== true) {
            die("执行失败！\r\n<br/>SQL：{$sql_list}\r\n<br/>error：{$res}");
        }

        return true;
    }

    // 获得所有分表缓存名称
    private function get_table_list() {
        // 判断缓存文件是否存在
        if (file_exists(self::$table_path)) {
            self::$table_list = require self::$table_path;
            return self::$table_list;
        }

        return [];
    }
    // 新增对外开放获取
    public function get_tables() {
        return $this->get_table_list();
    }

    // 创建表
    private function create_table($table)
    {
        // 获取缓存列表
        $this->get_table_list();

        $status = false;

        // 判断缓存文件是否存在
        if (!file_exists(self::$table_path)) {
            file_put_contents(self::$table_path, "<?php \r\nreturn [];");
        } else {
            // 判断是否存在缓存中
            foreach (self::$table_list as $v) {
                if ($v == $table) {
                    $status = true;
                    break;
                }
            }
        }

        // 为false则创建该表
        if (!$status) {
            // 先查下是否存在再说，防止并发或者跨服务器缓存不同步问题
            $sql = 'SHOW TABLES LIKE "'.self::$prefix.$table.'"';
            $result = $this->query( $sql);
            if (empty($result)){
                $sql = 'CREATE TABLE '.self::$prefix.$table.' LIKE '.self::$prefix.self::$table.';';
                $res = $this->exec($sql);
                if ($res !== true) {
                    die("{$table}：分表创建失败！\r\n<br/>SQL：{$sql}\r\n<br/>error：{$res}");
                }
            }
            
            // 加入分表中缓存
            self::$table_list[] = $table;
            // 更新缓存
            $html = "<?php \r\nreturn [\r\n";
            foreach (self::$table_list as $v) {
                $html .= '      "'.$v.'",'."\r\n";
            }
            $html .= "];";
            file_put_contents(self::$table_path, $html);
        }


        return $table;
    }

    /**
     * 执行查询语句
     * @todo 无
     * @author 小黄牛
     * @version v1.0.1 + 2019.05.20
     * @deprecated 暂不弃用
     * @global 无
     * @param string $sql
     * @return mixed
    */
    private function query($sql) {
        if (self::$Db) {
            $Db = self::$Db;
            $list = $Db->query($sql);
        } else {
            $list = \think\Db::query($sql);
        }
        return $list;
    }

    /**
     * 执行SQL语句
     * @todo 无
     * @author 小黄牛
     * @version v1.0.1 + 2019.05.20
     * @deprecated 暂不弃用
     * @global 无
     * @param string $name 表名
     * @return bool
    */
    private function exec($sql) {
        try {
            if (self::$Db) {
                $Db = self::$Db;
                $list = $Db->execute($sql);
            } else {
                $res = \think\Db::execute($sql);
            }

            if ($res !== false) return true;
        } catch(\PDOException $e) {
            return $e->getMessage();
        }
    }
}