<?php
// +----------------------------------------------------------------------
// | 消息回复助手类
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

namespace wx\sdk;

use wx\Weixins;
use think\facade\Request;

class message_reply extends Weixins{

    public static $block_size = 32;

    /**
     * 获取微信交互数据
     * @todo 无
     * @author 小黄牛
     * @version v1.0.0.4 + 2018.10.11
     * @deprecated 暂不弃用
     * @global 无
     * @return void
    */
    public static function xml(){
        self::get_config();
        # 获得数据包的信息
        $postStr = file_get_contents("php://input");
        \log\Log::init()->insert('请求接入：'.json_encode($postStr));
        # 如果数据包内的信息不为空
        if (!empty($postStr)) {
            // 开启加密模式
            if (self::$config['token_type'] == 2) {
                $postStr = self::decrypt($postStr);
            }
            # XML文件的解析依赖libxml库,libxml_disable_entity_loader函数,是为了安全性,防止入侵者通过协议注入XML向服务器发起攻击
            libxml_disable_entity_loader(true);
            # 把XML编译成一个Class对象
            $postObj = simplexml_load_string($postStr, 'SimpleXMLElement', LIBXML_NOCDATA);
            return $postObj;
        }
        return false;
    }

    
    /**
     * 输出数据到微信
     * @todo 无
     * @author 小黄牛
     * @version v1.0.0.4 + 2018.10.11
     * @deprecated 暂不弃用
     * @global 无
     * @return void
    */
    public static function send($xml){
        self::get_config();
        // 开启加密模式
        if (self::$config['token_type'] == 2) {
            $encrypt = self::encrypt($xml);
            $sTimeStamp = time();
            $sNonce = self::getRandomStr();
            $signature = self::getSHA1(self::$config['token'], $sTimeStamp, $sNonce, $encrypt);
            //生成发送的xml
            $xml = self::generate($encrypt, $signature, $sTimeStamp, $sNonce);
        }
        
        echo $xml;
        \log\Log::init()->insert('发送的XML：'.json_encode($xml));
        exit;
    }

    

    /**
     * 对明文进行加密
     * @param string $text 需要加密的明文
     * @return string 加密后的密文
     */
    private static function encrypt($text) {
        try {
            $EncodingAESKey = base64_decode(self::$config['EncodingAESKey'] . "=");
            //获得16位随机字符串，填充到明文之前
            $random =  self::getRandomStr();
            $text = $random . pack("N", strlen($text)) . $text . self::$config['appid'];
            $iv = substr($EncodingAESKey, 0, 16);
            $text = self::encode($text);
            $encrypted = openssl_encrypt($text,'AES-256-CBC',substr($EncodingAESKey, 0, 32),OPENSSL_ZERO_PADDING,$iv);
            return $encrypted;
        } catch (\Exception $e) {
            \log\Log::init()->insert('加密失败', 'error');
        }
    }
    
    /**
     * 对密文进行解密
     * @param string $encrypted 需要解密的密文
     * @return string 解密得到的明文
     */
    private static function decrypt($encrypted) {
        if (!function_exists('openssl_decrypt')) {
            \log\Log::init()->insert('解密失败-请先安装openssl_decrypt扩展与函数', 'error');
            return false;
        }
        try {
            $EncodingAESKey = base64_decode(self::$config['EncodingAESKey'] . "=");
            $iv = substr($EncodingAESKey, 0, 16);
            $array = self::extract($encrypted);
            if (!$array) {
                return false;
            }
            $decrypted = openssl_decrypt(base64_decode($array[1]),'AES-256-CBC', $EncodingAESKey, OPENSSL_RAW_DATA, $iv);
        } catch (\Exception $e) {
            \log\Log::init()->insert('解密失败-1', 'error');
        }
        try {
            //去除补位字符
            $result = self::decode($decrypted);
            //去除16位随机字符串,网络字节序和AppId
            if (strlen($result) < 16) {
                \log\Log::init()->insert('解密失败-2：', 'error');
                return false;
            }
            $content = substr($result, 16, strlen($result));
            $len_list = unpack("N", substr($content, 0, 4));
            $xml_len = $len_list[1];
            $xml_content = substr($content, 4, $xml_len);
            $from_appid = substr($content, $xml_len + 4);
        } catch (\Exception $e) {
            \log\Log::init()->insert('解密失败-3', 'error');
            return false;
        }
        if ($from_appid != self::$config['appid']) {
            \log\Log::init()->insert('解密失败-Appid错误', 'error');
            return false;
        }
        return $xml_content;
    }

    /**
     * 对需要加密的明文进行填充补位
     * @param $text 需要进行填充补位操作的明文
     * @return 补齐明文字符串
     */
    private static function encode($text) {
        $block_size = self::$block_size;
        $text_length = strlen($text);
        //计算需要填充的位数
        $amount_to_pad = self::$block_size - ($text_length % self::$block_size);
        if ($amount_to_pad == 0) {
            $amount_to_pad = self::$block_size;
        }
        //获得补位所用的字符
        $pad_chr = chr($amount_to_pad);
        $tmp = "";
        for ($index = 0; $index < $amount_to_pad; $index++) {
            $tmp .= $pad_chr;
        }
        return $text . $tmp;
    }

    /**
     * 对解密后的明文进行补位删除
     * @param decrypted 解密后的明文
     * @return 删除填充补位后的明文
     */
    private static function decode($text) {

        $pad = ord(substr($text, -1));
        if ($pad < 1 || $pad > 32) {
            $pad = 0;
        }
        return substr($text, 0, (strlen($text) - $pad));
    }

    
	/**
	 * 提取出xml数据包中的加密消息
	 * @param string $xmltext 待提取的xml字符串
	 * @return string 提取出的加密消息字符串
	 */
	private static function extract($xmltext) {
		try {
			$xml = new \DOMDocument();
			$xml->loadXML($xmltext);
			$array_e = $xml->getElementsByTagName('Encrypt');
			$array_a = $xml->getElementsByTagName('ToUserName');
			$encrypt = $array_e->item(0)->nodeValue;
			$tousername = $array_a->item(0)->nodeValue;
			return array(0, $encrypt, $tousername);
		} catch (\Exception $e) {
            \log\Log::init()->insert('XML提取失败', 'error');
            return false;
		}
	}

	/**
	 * 生成xml消息
	 * @param string $encrypt 加密后的消息密文
	 * @param string $signature 安全签名
	 * @param string $timestamp 时间戳
	 * @param string $nonce 随机字符串
	 */
	private static function generate($encrypt, $signature, $timestamp, $nonce) {
		$format = "<xml>
<Encrypt><![CDATA[%s]]></Encrypt>
<MsgSignature><![CDATA[%s]]></MsgSignature>
<TimeStamp>%s</TimeStamp>
<Nonce><![CDATA[%s]]></Nonce>
</xml>";
		return sprintf($format, $encrypt, $signature, $timestamp, $nonce);
    }
    
    /**
     * 随机生成16位字符串
     * @return string 生成的字符串
     */
    private static function getRandomStr() {
        $str = "";
        $str_pol = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789abcdefghijklmnopqrstuvwxyz";
        $max = strlen($str_pol) - 1;
        for ($i = 0; $i < 16; $i++) {
            $str .= $str_pol[mt_rand(0, $max)];
        }
        return $str;
    }

    /**
	 * 用SHA1算法生成安全签名
	 * @param string $token 票据
	 * @param string $timestamp 时间戳
	 * @param string $nonce 随机字符串
	 * @param string $encrypt 密文消息
	 */
	private static function getSHA1($token, $timestamp, $nonce, $encrypt_msg) {
		//排序
		try {
			$array = array($encrypt_msg, $token, $timestamp, $nonce);
			sort($array, SORT_STRING);
			$str = implode($array);
			return sha1($str);
		} catch (Exception $e) {
            \log\Log::init()->insert('sha1 签名生成失败', 'error');
            return false;
		}
	}
}